==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/symm/symm.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifdef XDOUBLE
#define ERROR_NAME "QSYMM "
#elif defined(DOUBLE)
#define ERROR_NAME "DSYMM "
#else
#define ERROR_NAME "SSYMM "
#endif

static int (*symm[])(long, long, long, FLOAT, FLOAT *, long, 
		FLOAT *, long, FLOAT *, long, FLOAT *) = {
#ifdef XDOUBLE
    qsymm_LU, qsymm_LL, qsymm_RU, qsymm_RL,
#elif defined(DOUBLE)
    dsymm_LU, dsymm_LL, dsymm_RU, dsymm_RL,
#else
    ssymm_LU, ssymm_LL, ssymm_RU, ssymm_RL,
#endif
};

int NAME(char *SIDE, char *UPLO, 
         blasint *M, blasint *N, 
         FLOAT *ALPHA, FLOAT *a, blasint *LDA, 
         FLOAT *b, blasint *LDB, 
         FLOAT *BETA,  FLOAT *c, blasint *LDC){
  
  char side_arg  = *SIDE;
  char uplo_arg  = *UPLO;

  blasint m = *M;
  blasint n = *N;
  FLOAT alpha = *ALPHA;
  FLOAT beta  = *BETA;
  blasint lda = *LDA;
  blasint ldb = *LDB;
  blasint ldc = *LDC;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

  blasint info;
  int side;
  int uplo;
  int nrowa;

  TOUPPER(side_arg);
  TOUPPER(uplo_arg);

  side  = -1;
  uplo  = -1;

  if (side_arg  == 'L') side  = 0;
  if (side_arg  == 'R') side  = 1;

  if (uplo_arg  == 'U') uplo  = 0;
  if (uplo_arg  == 'L') uplo  = 1;
  
  nrowa = m;
  if (side & 1) nrowa = n;

  info = 0;

  if (ldc < MAX(1,m))     info = 12;
  if (ldb < MAX(1,m))     info =  9;
  if (lda < MAX(1,nrowa)) info =  7;
  if (n < 0)              info =  4;
  if (m < 0)              info =  3;
  if (uplo  < 0)          info =  2;
  if (side  < 0)          info =  1;

  if (info != 0) {
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return 0;
  }
  
  if (m == 0 || n == 0) return 0;
  
#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP
  if (beta != ONE) GEMM_BETA(m, n, 0,  beta, NULL, 0, NULL, 0, c, ldc);
#else

#ifdef DOUBLE
  mode  =  BLAS_DOUBLE | BLAS_REAL;
#else
  mode  =  BLAS_SINGLE | BLAS_REAL;
#endif  

  if (beta != ONE)
    beta_thread(mode, m, n, beta, ZERO, c, ldc, (void *)GEMM_BETA);

#endif

  if (alpha == ZERO) return 0;
 
  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

  (symm[(side << 1) | uplo ])
    (m, n, 0, alpha, a, lda, b, ldb, c, ldc, buffer);

#else
  mode |= (side  << BLAS_RSIDE_SHIFT);

  symm_thread(mode, m, n, alpha, ZERO,
	      a, lda, b, ldb, c, ldc,
	      (void *)symm[(side << 1) | uplo ], buffer);

#endif

  blas_memory_free(buffer);

  return 0;
}

==> GotoBLAS-1.12.tar.gz: GotoBLAS/level3/symm/symm.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifdef XDOUBLE
#define ERROR_NAME "QSYMM "
#elif defined(DOUBLE)
#define ERROR_NAME "DSYMM "
#else
#define ERROR_NAME "SSYMM "
#endif

static int (*symm[])(long, long, long, FLOAT, FLOAT *, long, 
		FLOAT *, long, FLOAT *, long, FLOAT *) = {
#ifdef XDOUBLE
    qsymm_LU, qsymm_LL, qsymm_RU, qsymm_RL,
#elif defined(DOUBLE)
    dsymm_LU, dsymm_LL, dsymm_RU, dsymm_RL,
#else
    ssymm_LU, ssymm_LL, ssymm_RU, ssymm_RL,
#endif
};

int NAME(char *SIDE, char *UPLO, 
         blasint *M, blasint *N, 
         FLOAT *ALPHA, FLOAT *a, blasint *LDA, 
         FLOAT *b, blasint *LDB, 
         FLOAT *BETA,  FLOAT *c, blasint *LDC){
  
  char side_arg  = *SIDE;
  char uplo_arg  = *UPLO;

  blasint m = *M;
  blasint n = *N;
  FLOAT alpha = *ALPHA;
  FLOAT beta  = *BETA;
  blasint lda = *LDA;
  blasint ldb = *LDB;
  blasint ldc = *LDC;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

  blasint info;
  int side;
  int uplo;
  int nrowa;

  TOUPPER(side_arg);
  TOUPPER(uplo_arg);

  side  = -1;
  uplo  = -1;

  if (side_arg  == 'L') side  = 0;
  if (side_arg  == 'R') side  = 1;

  if (uplo_arg  == 'U') uplo  = 0;
  if (uplo_arg  == 'L') uplo  = 1;
  
  nrowa = m;
  if (side & 1) nrowa = n;

  info = 0;

  if (ldc < MAX(1,m))     info = 12;
  if (ldb < MAX(1,m))     info =  9;
  if (lda < MAX(1,nrowa)) info =  7;
  if (n < 0)              info =  4;
  if (m < 0)              info =  3;
  if (uplo  < 0)          info =  2;
  if (side  < 0)          info =  1;

  if (info != 0) {
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return 0;
  }
  
  if (m == 0 || n == 0) return 0;
  
#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP
  if (beta != ONE) GEMM_BETA(m, n, 0,  beta, NULL, 0, NULL, 0, c, ldc);
#else

#ifdef XDOUBLE
  mode  =  BLAS_XDOUBLE | BLAS_REAL;
#elif defined(DOUBLE)
  mode  =  BLAS_DOUBLE  | BLAS_REAL;
#else
  mode  =  BLAS_SINGLE  | BLAS_REAL;
#endif  

  if (beta != ONE)
    beta_thread(mode, m, n, beta, ZERO, c, ldc, (void *)GEMM_BETA);

#endif

  if (alpha == ZERO) return 0;
 
  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

  (symm[(side << 1) | uplo ])
    (m, n, 0, alpha, a, lda, b, ldb, c, ldc, buffer);

#else
  mode |= (side  << BLAS_RSIDE_SHIFT);

  symm_thread(mode, m, n, alpha, ZERO,
	      a, lda, b, ldb, c, ldc,
	      (void *)symm[(side << 1) | uplo ], buffer);

#endif

  blas_memory_free(buffer);

  return 0;
}
